@@ -485,8 +485,6 @@ gyrus_main_app_about (void)
 		NULL
 	};
 
-	const gchar *documentors[] = { NULL };
-
 	const gchar *translators = _("translators-credits");
 
 	logo_gyrus = g_strdup_printf ("%s%c%s", GYRUS_PIXMAPS_DIR,
@@ -503,7 +501,6 @@ gyrus_main_app_about (void)
 					      "(c) 2004-2005 Claudio Saavedra"),
 			       "comments", _("Administration tool for Cyrus IMAP servers."),
 			       "authors", authors, 
-			       "documenters", documentors,
 			       "translator-credits", strcmp (translators, "translators-credits") != 0 ? translators : NULL,
 			       "logo", pixbuf,
 			       NULL);
