@@ -42,6 +42,8 @@ enum
 	COLUMN_HOST,
 	COLUMN_USER,
 	COLUMN_PORT,
+	COLUMN_SEP_CHAR,
+	COLUMN_USETLS,
 	NUM_COLUMN
 };
 
@@ -65,9 +67,11 @@ static void gyrus_session_select_session (GyrusSession *session);
 static GConfClient *conf_client;
 static GtkWidget *window = NULL;
 static GtkWidget *window_edit = NULL;
+static GtkWidget *checkbutton_secure;
 static GtkWidget *entry_host;
 static GtkWidget *entry_user;
 static GtkWidget *entry_passwd;
+static GtkWidget *combobox_sep_char;
 static GtkWidget *button_new;
 static GtkWidget *button_open;
 static GtkWidget *button_delete;
@@ -81,6 +85,7 @@ static GtkTreeModel *model_sessions;
 static GtkWidget *button_session_edit_ok;
 static GtkWidget *button_session_edit_cancel;
 
+/*
 static void
 gyrus_session_debug_print_stored_sessions (GSList *sessions)
 {
@@ -91,6 +96,7 @@ gyrus_session_debug_print_stored_sessions (GSList *sessions)
 		g_print (": %s\n", tmp);
 	}
 }
+*/
 
 static void
 gyrus_session_list_free (GSList *sessions)
@@ -106,6 +112,17 @@ gyrus_session_list_free (GSList *sessions)
 	
 /*** module callbacks ***/
 
+void
+gyrus_session_checkbutton_secure_toggled (GtkWidget *widget,
+                                          gpointer data)
+{
+	g_return_if_fail (GTK_IS_SPIN_BUTTON (data));
+
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data),
+			gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)) ?
+				993 : 143);
+}
+
 void
 gyrus_session_edit_on_button_cancel_clicked (GtkWidget *widget,
 					     gpointer user_data)
@@ -194,31 +211,40 @@ gyrus_session_on_button_delete_session_clicked (GtkWidget *widget,
 
 	GSList *element = g_slist_find_custom (sessions, session->name,
 					       gyrus_session_name_compare_func);
-	if (element)
+	if (element) {
+		g_free (element->data);
 		sessions = g_slist_delete_link (sessions, element);
+	}
+
 	gconf_client_set_list (conf_client, key,
 			       GCONF_VALUE_STRING, sessions,
 			       NULL);
 	gyrus_session_list_free (sessions);
 	
-	gchar *host, *port, *user, *path;
+	gchar *host, *port, *user, *path, *sep_char, *usetls;
 	
 	host = g_strconcat (PATH_SESSIONS, session->name, "/", "Host", NULL);
 	port = g_strconcat (PATH_SESSIONS, session->name, "/", "Port", NULL);
 	user = g_strconcat (PATH_SESSIONS, session->name, "/", "User", NULL);
+	usetls = g_strconcat (PATH_SESSIONS, session->name, "/", "UseTLS", NULL);
 	path = g_strconcat (PATH_SESSIONS, session->name, NULL);
+	sep_char = g_strconcat (PATH_SESSIONS, session->name, "/", "SeparatorChar", NULL);
 	
 	gconf_client_unset (conf_client, host, NULL);
 	gconf_client_unset (conf_client, port, NULL);
 	gconf_client_unset (conf_client, user, NULL);
+	gconf_client_unset (conf_client, usetls, NULL);
 	gconf_client_unset (conf_client, path, NULL);
+	gconf_client_unset (conf_client, sep_char, NULL);
 	
 	g_free (key);
 	g_free (path);
 	g_free (host);
+	g_free (usetls);
 	g_free (port);
 	g_free (user);
-	
+	g_free (sep_char);
+
 	gyrus_session_list_set_sensitive (FALSE);
 	gyrus_session_free (session);
 }
@@ -241,19 +267,24 @@ gyrus_session_list_set_sensitive (gboolean sensitive)
 static void
 gyrus_session_edit_session (GyrusSession *session)
 {
-	gchar *host, *port, *user;
+	gchar *host, *port, *user, *sep_char, *usetls;
 	
 	host = g_strconcat (PATH_SESSIONS, session->name, "/", "Host", NULL);
 	port = g_strconcat (PATH_SESSIONS, session->name, "/", "Port", NULL);
 	user = g_strconcat (PATH_SESSIONS, session->name, "/", "User", NULL);
-	
+	sep_char = g_strconcat (PATH_SESSIONS, session->name, "/", "SeparatorChar", NULL);
+	usetls = g_strconcat (PATH_SESSIONS, session->name, "/", "UseTLS", NULL);
 	gconf_client_set_string (conf_client, host, session->host, NULL);
 	gconf_client_set_int (conf_client, port, session->port, NULL);
 	gconf_client_set_string (conf_client, user, session->user, NULL);
+	gconf_client_set_string (conf_client, sep_char, session->sep_char, NULL);
+	gconf_client_set_bool (conf_client, usetls, session->usetls, NULL);
 
 	g_free (host);
 	g_free (port);
 	g_free (user);
+	g_free (sep_char);
+	g_free (usetls);
 
 	gyrus_session_load_gconf_settings();
 }
@@ -261,7 +292,7 @@ gyrus_session_edit_session (GyrusSession *session)
 gboolean
 gyrus_session_is_valid (GyrusSession *session, gchar **why_invalid)
 {
-	gchar *host, *user, *path;
+	gchar *host, *user, *path, *sep_char, *usetls;
 
 	gboolean is_valid;
 
@@ -274,12 +305,21 @@ gyrus_session_is_valid (GyrusSession *session, gchar **why_invalid)
 	host = g_strconcat (PATH_SESSIONS, session->name, "/", "Host", NULL);
 	is_valid = gconf_valid_key (host, why_invalid);
 	g_free (host);
-
 	g_return_val_if_fail (is_valid, FALSE);
 
 	user = g_strconcat (PATH_SESSIONS, session->name, "/", "User", NULL);
-	gconf_valid_key (host, why_invalid);
+	is_valid = gconf_valid_key (user, why_invalid);
 	g_free (user);
+	g_return_val_if_fail (is_valid, FALSE);
+
+	sep_char = g_strconcat (PATH_SESSIONS, session->name, "/", "SeparatorChar", NULL);
+	is_valid = gconf_valid_key (sep_char, why_invalid);
+	g_free (sep_char);
+	g_return_val_if_fail (is_valid, FALSE);
+
+	usetls = g_strconcat (PATH_SESSIONS, session->name, "/", "UseTLS", NULL);
+	is_valid = gconf_valid_key (usetls, why_invalid);
+	g_free (usetls);
 
 	return is_valid;
 }
@@ -289,7 +329,8 @@ gyrus_session_append_session (GyrusSession *new)
 {
 	g_return_if_fail (new != NULL);
 
-	gchar *host, *port, *user;
+	gchar *host, *port, *user, *sep_char, *usetls;
+
 	gchar *name;
 
 	if (new->name == NULL)
@@ -328,14 +369,20 @@ gyrus_session_append_session (GyrusSession *new)
 	host = g_strconcat (PATH_SESSIONS, new->name, "/", "Host", NULL);
 	port = g_strconcat (PATH_SESSIONS, new->name, "/", "Port", NULL);
 	user = g_strconcat (PATH_SESSIONS, new->name, "/", "User", NULL);
+	sep_char = g_strconcat (PATH_SESSIONS, new->name, "/", "SeparatorChar", NULL);
+	usetls = g_strconcat (PATH_SESSIONS, new->name, "/", "UseTLS", NULL);
 	
 	gconf_client_set_string (conf_client, host, new->host, NULL);
 	gconf_client_set_int    (conf_client, port, new->port, NULL);
 	gconf_client_set_string (conf_client, user, new->user, NULL);
+	gconf_client_set_string (conf_client, sep_char, new->sep_char, NULL);
+	gconf_client_set_bool   (conf_client, usetls, new->usetls, NULL);
 
 	g_free (host);
 	g_free (port);
 	g_free (user);
+	g_free (sep_char);
+	g_free (usetls);
 	
 	gyrus_session_load_gconf_settings();
 	gyrus_common_show_message (GTK_WINDOW (window), GTK_MESSAGE_INFO,
@@ -347,11 +394,26 @@ gyrus_session_set_defaults_from_session (GyrusSession *session)
 {
 	g_return_if_fail (session);
 
+	gint value = 0;
+
 	gtk_entry_set_text (GTK_ENTRY (entry_host), session->host);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (spinbutton_port),
 				   (gdouble) session->port);
 	gtk_entry_set_text (GTK_ENTRY (entry_user), session->user);
 	gtk_entry_set_text (GTK_ENTRY (entry_name), session->name);
+	
+	if (strcmp (session->sep_char, ".") == 0)
+		value = 0;
+	else if (strcmp (session->sep_char, "/") == 0)
+		value = 1;
+	else if (strcmp (session->sep_char, "auto") == 0)
+		value = 2;
+
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_sep_char), 
+				  value);
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton_secure),
+	                              session->usetls);
 }
 
 GyrusSession *
@@ -382,7 +444,16 @@ gyrus_session_new_from_interface (void)
 	else
 		session->passwd = NULL;
 
-
+	session->sep_char = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combobox_sep_char));
+	if (!g_utf8_strlen (session->sep_char, -1))
+		session->sep_char = g_strdup (".");
+	else if (strcmp (session->sep_char, _("Autodetect")) == 0) {
+		g_free (session->sep_char);
+		session->sep_char = g_strdup ("auto");
+	}
+	
+	session->usetls = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton_secure));
+	
 	return session;
 }
 
@@ -398,6 +469,8 @@ gyrus_session_add_session_to_treeview (GyrusSession *session)
 			    COLUMN_HOST, session->host,
 			    COLUMN_USER, session->user,
 			    COLUMN_PORT, session->port,
+			    COLUMN_SEP_CHAR, session->sep_char,
+			    COLUMN_USETLS, session->usetls,
 			    -1);
 }
 
@@ -419,12 +492,12 @@ gyrus_session_treeview_on_key_press_event (GtkWidget *widget,
 {
 	GtkTreeSelection *selection;
 	GtkTreeView *treeview = GTK_TREE_VIEW (widget);
-
+	
 	selection = gtk_tree_view_get_selection (treeview);
 	
 	if (gtk_tree_selection_count_selected_rows (selection) == 0)
 		return FALSE;
-
+	
 	switch (event->keyval) {
 	case GDK_Delete:
 		gtk_button_clicked (GTK_BUTTON (button_delete));
@@ -462,7 +535,7 @@ gyrus_session_set_treeview (void)
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *selection;
-
+	
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (_("Session"),
 							   renderer,
@@ -477,7 +550,7 @@ gyrus_session_set_treeview (void)
 	gtk_tree_selection_set_select_function (selection, 
 						gyrus_session_on_selection_changed,
 						NULL, NULL);
-
+	
 	g_signal_connect (G_OBJECT (treeview_sessions), "key-press-event",
 			  G_CALLBACK (gyrus_session_treeview_on_key_press_event),
 			  NULL);
@@ -487,67 +560,89 @@ static void
 gyrus_session_create_model_sessions (void)
 {
 	GtkListStore *model;
-
+	
 	model = gtk_list_store_new (NUM_COLUMN,
 				    G_TYPE_STRING,
 				    G_TYPE_STRING,
 				    G_TYPE_STRING,
-				    G_TYPE_INT);
+				    G_TYPE_INT,
+				    G_TYPE_STRING,
+				    G_TYPE_BOOLEAN);
 	
 	model_sessions = GTK_TREE_MODEL (model);
 }
 
 /* If remove is TRUE, the select item is remove from the array */
- GyrusSession *
+GyrusSession *
 gyrus_session_get_selected_session (gboolean remove)
 {
-	GtkTreeIter iter;
+GtkTreeIter iter;
 	GtkTreeSelection *selection;
 
 	GyrusSession *session = g_new0 (GyrusSession, 1);
 	
 	selection =
 		gtk_tree_view_get_selection(GTK_TREE_VIEW (treeview_sessions));
-
+	
 	if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
 		return NULL;
-
+	
 	gtk_tree_model_get (model_sessions, &iter,
 			    COLUMN_NAME, &(session->name),
 			    COLUMN_HOST, &(session->host),
 			    COLUMN_USER, &(session->user),
 			    COLUMN_PORT, &(session->port),
+			    COLUMN_SEP_CHAR, &(session->sep_char),
+			    COLUMN_USETLS, &(session->usetls),
 			    -1);
-
+	
 	if (remove) {
 		gtk_list_store_remove (GTK_LIST_STORE (model_sessions),
 				       &iter);
 	}
-
+	
 	return session;
 }
 
 static GyrusSession *
 gyrus_session_get_with_name (const gchar *name)
 {
+	GError *error = NULL;
 	GyrusSession *aux = NULL;
 	gchar *host;
 	gchar *port;
 	gchar *user;
-
+	gchar *sep_char;
+	gchar *usetls;
+	
 	aux = g_new0 (GyrusSession, 1);
 	host = g_strconcat (PATH_SESSIONS, name, "/", "Host", NULL);
 	port = g_strconcat (PATH_SESSIONS, name, "/", "Port", NULL);
 	user = g_strconcat (PATH_SESSIONS, name, "/", "User", NULL);
+	sep_char = g_strconcat (PATH_SESSIONS, name, "/", "SeparatorChar", NULL);
+	usetls = g_strconcat (PATH_SESSIONS, name, "/", "UseTLS", NULL);
 	
 	aux->name = g_strdup (name);
 	aux->host = gconf_client_get_string (conf_client, host, NULL);
 	aux->port = gconf_client_get_int (conf_client, port, NULL);
 	aux->user = gconf_client_get_string (conf_client, user, NULL);
+	aux->sep_char = gconf_client_get_string (conf_client, sep_char, 
+						 &error);
+	if (aux->sep_char == NULL)
+		aux->sep_char = g_strdup (".");
+	else if (error) {
+		aux->sep_char = g_strdup (".");
+		g_warning (error->message);
+		g_error_free (error);
+	}
+
+	aux->usetls = gconf_client_get_bool (conf_client, usetls, NULL);
 
 	g_free (host);
 	g_free (port);
 	g_free (user);
+	g_free (sep_char);
+	g_free (usetls);
 
 	return aux;
 }
@@ -667,9 +762,11 @@ gyrus_session_edit_load_interface (gpointer user_data)
 	g_free (file_glade);
 	
 	window_edit       = glade_xml_get_widget (xml, "dialog_session_edit");
+	checkbutton_secure= glade_xml_get_widget (xml, "checkbutton_secure");
 	entry_host        = glade_xml_get_widget (xml, "entry_host");
 	entry_user        = glade_xml_get_widget (xml, "entry_user");
 	entry_passwd      = glade_xml_get_widget (xml, "entry_passwd");
+	combobox_sep_char      = glade_xml_get_widget (xml, "combobox_sep_char");
 	spinbutton_port   = glade_xml_get_widget (xml, "spinbutton_port");
 	button_session_edit_ok  = glade_xml_get_widget (xml, "button_session_edit_ok");
 	button_session_edit_cancel = glade_xml_get_widget (xml, "button_session_edit_cancel");
@@ -678,6 +775,10 @@ gyrus_session_edit_load_interface (gpointer user_data)
 	g_signal_connect (G_OBJECT (button_session_edit_cancel), "clicked",
 			  G_CALLBACK (gyrus_session_edit_on_button_cancel_clicked),
 			  NULL);
+
+	g_signal_connect (G_OBJECT (checkbutton_secure), "toggled",
+	                  G_CALLBACK (gyrus_session_checkbutton_secure_toggled),
+	                  (gpointer) spinbutton_port);
 	
 	gtk_window_set_modal (GTK_WINDOW (window_edit), TRUE);
 	gtk_window_set_transient_for (GTK_WINDOW (window_edit), GTK_WINDOW (window));
@@ -687,7 +788,8 @@ gyrus_session_edit_load_interface (gpointer user_data)
 
 /* show window sessions */
 void 
-gyrus_session_show_sessions (GtkWidget *widget, gpointer user_data)
+gyrus_session_show_sessions (GtkWidget *widget G_GNUC_UNUSED,
+			     gpointer user_data)
 {
 	GtkTreeIter iter;
 	conf_client = gconf_client_get_default ();
@@ -761,6 +863,9 @@ gyrus_session_free (GyrusSession *session)
 		g_free (session->host);
 	if (session->passwd)
 		g_free (session->passwd);
+	if (session->sep_char)
+		g_free (session->sep_char);
+
 	g_free (session);
 	session = NULL;
 }
